@@ -34,6 +34,7 @@ Card: Highscreen Sound-Boostar 16 3D\n\
 ISAPNP: MDK1605=ADS7180,ADS7181\n\
 ISAPNP: ADS7150=ADS7150,ADS7151\n\
 ISAPNP: ADS7181=ADS7180,ADS7181\n\
+ISAPNP: TER1411=ADS7180,ADS7181\n\
 ");
 
 int snd_index[SND_CARDS] = SND_DEFAULT_IDX;		/* Index 1-MAX */
@@ -103,6 +104,10 @@ static unsigned int snd_card_ad1816a_pnpids[] = {
 	(ISAPNP_VENDOR('A','D','S')<<16)|ISAPNP_DEVICE(0x7181),    /* device */
 	(ISAPNP_VENDOR('A','D','S')<<16)|ISAPNP_FUNCTION(0x7180),  /* audio */
 	(ISAPNP_VENDOR('A','D','S')<<16)|ISAPNP_FUNCTION(0x7181),  /* mpu401 */
+	/* Analog Devices AD1816A - Terratec Base 64 */
+	(ISAPNP_VENDOR('T','E','R')<<16)|ISAPNP_DEVICE(0x1411),    /* device */
+	(ISAPNP_VENDOR('A','D','S')<<16)|ISAPNP_FUNCTION(0x7180),  /* audio */
+	(ISAPNP_VENDOR('A','D','S')<<16)|ISAPNP_FUNCTION(0x7181),  /* mpu401 */
 	0	/* end */
 };
 #endif	/* __ISAPNP__ */
