@@ -50,6 +50,7 @@ int snd_dma1[SND_CARDS] = SND_DEFAULT_DMA;	/* 1,3,5,6,7 */
 int snd_dma2[SND_CARDS] = SND_DEFAULT_DMA;	/* 1,3,5,6,7 */
 int snd_dma1_size[SND_CARDS] = SND_DEFAULT_DMA_SIZE;	/* 8,16,32,64,128 */
 int snd_dma2_size[SND_CARDS] = SND_DEFAULT_DMA_SIZE;	/* 8,16,32,64,128 */
+int snd_opl3sa3_ymode[SND_CARDS] = { [0 ... (SND_CARDS-1)] = 0 };   /* 0,1,2,3 */ /*SL Added*/
 #ifdef __ISAPNP__
 int snd_isapnp[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = 1};
 #endif
@@ -79,6 +80,8 @@ MODULE_PARM(snd_dma2_size, "1-" __MODULE_STRING(SND_CARDS) "i");
 MODULE_PARM_DESC(snd_dma2_size, "DMA2 size in kB for OPL3-SA driver. [DMA_SIZE]");
 MODULE_PARM(snd_isapnp, "1-" __MODULE_STRING(SND_CARDS) "i");
 MODULE_PARM_DESC(snd_isapnp, "ISA PnP detection for specified soundcard. [BOOL]");
+MODULE_PARM(snd_opl3sa3_ymode, "1-" __MODULE_STRING(SND_CARDS) "i"); /* SL Added */
+MODULE_PARM_DESC(snd_opl3sa3_ymode, "Speaker size selection for 3D Enhancement mode: Desktop/Large Notebook/Small Notebook/HiFi. [list=0,1,2,3]");  /* SL Added */
 
 struct snd_opl3sa {
 	int version;		/* 2 or 3 */
@@ -101,6 +104,8 @@ struct snd_opl3sa {
 #ifdef __ISAPNP__
 	struct isapnp_dev *dev;
 #endif
+	int ymode;	/* SL added*/
+	int wide;
 };
 
 static struct snd_opl3sa *snd_opl3sa_cards[SND_CARDS] = SND_DEFAULT_PTR;
@@ -236,7 +241,11 @@ static int __init snd_opl3sa_detect(struct snd_opl3sa *oplcard,
 	snd_opl3sa_write(port, 0x09, 0x9f);
 	/* initialization */
 	snd_opl3sa_write(port, 0x01, 0x00);	/* Power Management - default */
-	snd_opl3sa_write(port, 0x02, 0x00);	/* System Control - default */
+	if (oplcard->version > 2) {   /* SL Added */
+		snd_opl3sa_write(port, 0x02, (oplcard->ymode << 4)); /* SL Modified - System Control - ymode is bits 4&5 (of 0 to 7) on all but opl3sa2 versions */
+	} else { /* SL Added */
+		snd_opl3sa_write(port, 0x02, 0x00);	/* SL Modified - System Control - default for opl3sa2 versions */
+	} /* SL Added */
 	snd_opl3sa_write(port, 0x03, 0x0d);	/* Interrupt Channel Configuration - IRQ A = OPL3 + MPU + WSS */
 	if (oplcard->dma2ptr == NULL) {
 		snd_opl3sa_write(port, 0x06, 0x03);	/* DMA Configuration - DMA A = WSS-R + WSS-P */
@@ -314,6 +323,7 @@ static int __init snd_opl3sa_resources(int dev, struct snd_opl3sa *oplcard, snd_
 	} else {
 		oplcard->dma2ptr = NULL;
 	}
+	oplcard->ymode = snd_opl3sa3_ymode[dev] & 0x03 ; /* SL Added - initialise this card from supplied (or default) parameter*/ 
 	return 0;
 }
 
@@ -465,11 +475,12 @@ static int snd_opl3sa_3d_control(snd_kmixer_element_t * element,
 	int change = 0;
 	
 	spin_lock_irqsave(&oplcard->reg_lock, flags);
-	owide = snd_opl3sa_read(oplcard->port, 0x14) & 7;
+	owide = oplcard->wide & 7;
 	if (w_flag) {
 		if (eff->effect & SND_MIXER_EFF1_WIDE) {
 			change |= owide != eff->wide;
-			owide = ((eff->wide & 7) << 4) | (eff->wide & 7);
+			oplcard->wide =
+			  owide = ((eff->wide & 7) << 4) | (eff->wide & 7);
 			snd_opl3sa_write(oplcard->port, 0x14, owide);
 		}
 	} else {
