@@ -121,18 +121,15 @@ static unsigned int snd_card_opti9xx_pnpids[] = {
 #ifndef OPTi93X
 	/* OPTi 82C924 */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_DEVICE(0x0924)),    /* device */
-	OPTi9XX_HW_82C924,					   /* chip */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_FUNCTION(0x0000)),  /* audio */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_FUNCTION(0x0002)),  /* mpu401 */
 	/* OPTi 82C925 */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_DEVICE(0x0925)),    /* device */
-	OPTi9XX_HW_82C925,					   /* chip */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_FUNCTION(0x9250)),  /* audio */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_FUNCTION(0x0002)),  /* mpu401 */
 #else
 	/* OPTi 82C931/3 */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_DEVICE(0x0931)),    /* device */
-	OPTi9XX_HW_82C931,					   /* chip */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_FUNCTION(0x9310)),  /* audio */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_FUNCTION(0x0002)),  /* mpu401 */
 #endif	/* OPTi93X */
@@ -277,19 +274,20 @@ __again:
 	while ((tmp = snd_card_opti9xx_pnpids[idx]) != 0) {
 		if ((card = isapnp_find_card(tmp >> 16, tmp & 0xffff, card)))
 			break;
-		idx += 4;
+		idx += 3;
 	}
 	if (card == NULL)
 		return -ENODEV;
 
-	hw = snd_card_opti9xx_pnpids[idx+1];
-	tmp = snd_card_opti9xx_pnpids[idx+2];
+	hw = tmp & 0xffff;
+
+	tmp = snd_card_opti9xx_pnpids[idx+1];
 	pdev = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
 	if (pdev == NULL)
 		goto __again;
 
 	acard->dev = pdev;
-	tmp = snd_card_opti9xx_pnpids[idx+3];
+	tmp = snd_card_opti9xx_pnpids[idx+2];
 	acard->devmpu = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
 
 	pdev = acard->dev;
@@ -300,7 +298,7 @@ __again:
 	if (snd_port != SND_AUTO_PORT)
 		isapnp_resource_change(&pdev->resource[0], snd_port + 4, 4);
 #else
-	if ((hw != OPTi9XX_HW_82C924) && (snd_port != SND_AUTO_PORT))
+	if ((hw != 0x0924) && (snd_port != SND_AUTO_PORT))
 		isapnp_resource_change(&pdev->resource[1], snd_port, 4);
 #endif	/* OPTi93X */
 	if (snd_irq != SND_AUTO_IRQ)
@@ -323,7 +321,7 @@ __again:
 	snd_port = pdev->resource[0].start - 4;
 	snd_fm_port = pdev->resource[1].start;
 #else
-	if (hw != OPTi9XX_HW_82C924)
+	if (hw != 0x0924)
 		snd_port = pdev->resource[1].start;
 	snd_fm_port = pdev->resource[2].start;
 #endif	/* OPTi93X */
@@ -496,7 +494,22 @@ static int __init snd_card_opti9xx_probe(struct snd_card_opti9xx *acard)
 #endif	/* OPTi93X */
 
 #ifdef __ISAPNP__
-	if ((hw = snd_card_opti9xx_isapnp(acard)) >= 0) {
+	if ((hw = snd_card_opti9xx_isapnp(acard)) > 0) {
+		switch (hw) {
+		case ISAPNP_DEVICE(0x0924):
+			hw = OPTi9XX_HW_82C924;
+			break;
+		case ISAPNP_DEVICE(0x0925):
+			hw = OPTi9XX_HW_82C925;
+			break;
+		case ISAPNP_DEVICE(0x0931):
+			hw = OPTi9XX_HW_82C931;
+			break;
+		default:
+			error = -ENODEV;
+			goto __on_error;
+		}
+
 		if (!(chip = snd_opti9xx_new_device(hw))) {
 			error = -ENOMEM;
 			goto __on_error;
