@@ -35,8 +35,12 @@
 #define NO_FILL_ROUTINE
 #endif
 
-/*
+#if LinuxVersionCode(2, 3, 48) <= LINUX_VERSION_CODE
+#define NEW_INODE_OPS
+#endif
 
+/*
+ *
  */
 
 int snd_info_check_reserved_words(const char *str)
@@ -556,6 +560,7 @@ static struct file_operations snd_info_entry_operations =
 	release:	snd_info_entry_release,
 };
 
+#ifndef NEW_INODE_OPS
 static struct inode_operations snd_info_entry_inode_operations =
 {
 	&snd_info_entry_operations,	/* default sound info directory file-ops */
@@ -567,6 +572,7 @@ static struct inode_operations snd_info_device_inode_operations =
 	&snd_fops,		/* default sound info directory file-ops */
 };
 #endif
+#endif	/* NEW_INODE_OPS */
 
 static int snd_info_card_readlink(struct dentry *dentry,
 				  char *buffer, int buflen)
@@ -607,14 +613,18 @@ static struct dentry *snd_info_card_followlink(struct dentry *dentry,
 	return lookup_dentry(tmp, base, follow);
 }
 
+#ifndef NEW_INODE_OPS
 static struct file_operations snd_info_card_link_operations =
 {
 	NULL
 };
+#endif
 
 struct inode_operations snd_info_card_link_inode_operations =
 {
+#ifndef NEW_INODE_OPS
 	default_file_ops:	&snd_info_card_link_operations,
+#endif
 	readlink:		snd_info_card_readlink,
 	follow_link:		snd_info_card_followlink,
 };
@@ -709,7 +719,11 @@ int snd_info_card_register(snd_card_t * card)
 		return -ENOMEM;
 	}
 	snd_info_entry_prepare(p);
+#ifdef NEW_INODE_OPS
+	p->proc_iops = &snd_info_card_link_inode_operations;
+#else
 	p->ops = &snd_info_card_link_inode_operations;
+#endif
 	card->proc_dir_link = p;
 	return 0;
 }
@@ -880,7 +894,11 @@ snd_info_entry_t *snd_info_create_device(const char *name, unsigned int number,
 	p = create_proc_entry(entry->name, entry->mode, snd_proc_dev);
 	if (p) {
 		snd_info_device_entry_prepare(p, entry);
+#ifdef NEW_INODE_OPS
+		p->proc_fops = &snd_fops;
+#else
 		p->ops = &snd_info_device_inode_operations;
+#endif
 	} else {
 		up(&info_mutex);
 		snd_info_free_entry(entry);
@@ -982,7 +1000,11 @@ int snd_info_register(snd_info_entry_t * entry)
 	p = create_proc_entry(entry->name, entry->mode, root);
 	if (p) {
 		snd_info_entry_prepare(p);
+#ifdef NEW_INODE_OPS
+		p->proc_fops = &snd_info_entry_operations;
+#else
 		p->ops = &snd_info_entry_inode_operations;
+#endif
 		p->size = entry->size;
 	} else {
 		up(&info_mutex);
