@@ -75,6 +75,7 @@ snd_card_t *snd_card_new(int idx, char *xid,
 		spin_unlock_irqrestore(&snd_card_lock, flags);
 		snd_kfree(card);
 		MOD_DEC_USE_COUNT;
+		snd_printk("card is out of range (0-%i)\n", snd_ecards_limit-1);
 		return NULL;
 	}
 	snd_cards_lock |= 1 << idx;		/* lock it */
@@ -85,7 +86,8 @@ snd_card_t *snd_card_new(int idx, char *xid,
 	card->use_inc = use_inc;
 	card->use_dec = use_dec;
 	init_MUTEX(&card->control);
-	snd_switch_prepare(&card->switches);
+	init_MUTEX(&card->lists_lock);
+	snd_switch_prepare(card, &card->switches, card, SND_CTL_IFACE_CONTROL, 0, 0);
 	/* control interface cannot be accessed from user space until */
 	/* snd_cards_bitmask and snd_cards are set by snd_card_register */
 	if (snd_control_register(card) < 0) {
@@ -106,7 +108,8 @@ int snd_card_free(snd_card_t * card)
 {
 	unsigned long flags;
 
-	snd_debug_check(card == NULL, -ENXIO);
+	if (card == NULL)
+		return -ENXIO;
 	if (snd_device_free_all(card) < 0) {
 		snd_printk("unable to free all devices\n");
 		/* Fatal, but this situation should never occur */
@@ -119,7 +122,7 @@ int snd_card_free(snd_card_t * card)
 		snd_printk("unable to unregister control minors\n");
 		/* Not fatal error */
 	}
-	snd_switch_free(&card->switches);
+	snd_switch_free(card, &card->switches);
 	spin_lock_irqsave(&snd_card_lock, flags);
 	snd_cards_lock &= ~(1 << card->number);
 	spin_unlock_irqrestore(&snd_card_lock, flags);
@@ -138,7 +141,8 @@ int snd_card_register(snd_card_t * card)
 	unsigned long flags;
 	int err;
 
-	snd_debug_check(card == NULL, -ENXIO);
+	if (card == NULL)
+		return -ENXIO;
 	if ((err = snd_device_register_all(card)) < 0)
 		return err;
 	spin_lock_irqsave(&snd_card_lock, flags);
@@ -153,7 +157,8 @@ int snd_card_unregister(snd_card_t * card)
 {
 	unsigned long flags;
 
-	snd_debug_check(card == NULL, -ENXIO);
+	if (card == NULL)
+		return -ENXIO;
 	spin_lock_irqsave(&snd_card_lock, flags);
 	snd_cards_bitmap &= ~(1 << card->number);
 	snd_cards[card->number] = NULL;
