@@ -728,34 +728,6 @@ int snd_mixer_group_element_remove(snd_kmixer_t * mixer,
 	return -ENXIO;
 }
 
-static int snd_mixer_switch_read(snd_kmixer_t * mixer,
-				 snd_switch_t * _uswitch)
-{
-	return snd_switch_read(&mixer->switches, mixer, _uswitch);
-}
-
-static int snd_mixer_switch_write(snd_control_t * control,
-				  snd_kmixer_t * mixer,
-				  snd_switch_t * _uswitch)
-{
-	int result;
-
-	if (snd_control_busy(control))
-		return -EBUSY;
-	result = snd_switch_write(&mixer->switches, mixer, _uswitch);
-	if (result > 0) {	/* change */
-		snd_switch_t uswitch;
-
-		if (copy_from_user(&uswitch, _uswitch, sizeof(uswitch)))
-			return -EFAULT;
-		snd_control_notify_switch_value_change(control,
-						       SND_CTL_IFACE_MIXER,
-						       uswitch.name,
-						       0);
-	}
-	return result;
-}
-
 static int snd_mixer_info(snd_kmixer_t * mixer, snd_mixer_info_t * _info)
 {
 	snd_mixer_info_t info;
@@ -1123,21 +1095,12 @@ static int snd_mixer_control_ioctl(snd_card_t * card, snd_control_t * control, u
 			return 0;
 		}
 	case SND_CTL_IOCTL_MIXER_INFO:
-	case SND_CTL_IOCTL_MIXER_SWITCH_LIST:
-	case SND_CTL_IOCTL_MIXER_SWITCH_READ:
-	case SND_CTL_IOCTL_MIXER_SWITCH_WRITE:
 		mixer = snd_mixers[tmp + control->mixer_device];
 		if (mixer == NULL)
 			return -ENXIO;
 		switch (cmd) {
 		case SND_CTL_IOCTL_MIXER_INFO:
 			return snd_mixer_info(mixer, (snd_mixer_info_t *) arg);
-		case SND_CTL_IOCTL_MIXER_SWITCH_LIST:
-			return snd_switch_list(&mixer->switches, (snd_switch_list_t *) arg);
-		case SND_CTL_IOCTL_MIXER_SWITCH_READ:
-			return snd_mixer_switch_read(mixer, (snd_switch_t *) arg);
-		case SND_CTL_IOCTL_MIXER_SWITCH_WRITE:
-			return snd_mixer_switch_write(control, mixer, (snd_switch_t *) arg);
 		}
 		break;
 	}
@@ -1264,7 +1227,7 @@ int snd_mixer_new(snd_card_t * card, char *id, int device, snd_kmixer_t ** rmixe
 	}
 	mixer->ffile_lock = SPIN_LOCK_UNLOCKED;
 	init_MUTEX(&mixer->lock);
-	snd_switch_prepare(&mixer->switches);
+	snd_switch_prepare(card, &mixer->switches, mixer, SND_CTL_IFACE_MIXER, mixer->device, 0);
 	if ((err = snd_device_new(card, SND_DEV_MIXER, mixer, device, &ops, NULL)) < 0) {
 		snd_mixer_free(mixer);
 		return err;
@@ -1289,7 +1252,7 @@ static int snd_mixer_free(snd_kmixer_t * mixer)
 		mixer->groups = group->next;
 		snd_mixer_group_free(group);
 	}
-	snd_switch_free(&mixer->switches);
+	snd_switch_free(mixer->card, &mixer->switches);
 	if (mixer->private_data && mixer->private_free)
 		mixer->private_free(mixer->private_data);
 	snd_magic_kfree(mixer);
@@ -1303,6 +1266,7 @@ int snd_mixer_switch_add(snd_kmixer_t * mixer, snd_kswitch_t * kswitch)
 		snd_control_notify_switch_change(mixer->card,
 						 SND_CTL_READ_SWITCH_ADD,
 						 SND_CTL_IFACE_MIXER,
+						 mixer->device, 0,
 						 kswitch->name);
 	return err;
 }
@@ -1316,6 +1280,7 @@ int snd_mixer_switch_remove(snd_kmixer_t * mixer, snd_kswitch_t * kswitch)
 		snd_control_notify_switch_change(mixer->card,
 						 SND_CTL_READ_SWITCH_REMOVE,
 						 SND_CTL_IFACE_MIXER,
+						 mixer->device, 0,
 						 kswitch->name);
 	return err;
 }
@@ -1341,6 +1306,7 @@ int snd_mixer_switch_change(snd_kmixer_t * mixer, snd_kswitch_t * kswitch)
 	snd_control_notify_switch_change(mixer->card,
 					 SND_CTL_READ_SWITCH_CHANGE,
 					 SND_CTL_IFACE_MIXER,
+					 mixer->device, 0,
 					 kswitch->name);
 	return 0;
 }
