@@ -44,27 +44,6 @@ void snd_pcm_lock(int xup)
 	}
 }
 
-static int snd_pcm_switch_write(snd_control_t * control, snd_pcm_t * pcm, int iface,
-				snd_kswitch_list_t * klist, snd_switch_t *_sw)
-{
-	int result;
-	
-	if (snd_control_busy(control))
-		return -EBUSY;
-	result = snd_switch_write(klist, pcm, _sw);
-	if (result > 0) {	/* change */
-		snd_switch_t sw;
-		
-		if (copy_from_user(&sw, _sw, sizeof(*_sw)))
-			return -EFAULT;
-		snd_control_notify_switch_value_change(control,
-						       iface,
-						       sw.name,
-						       0);
-	}
-	return result;
-}
-
 static int snd_pcm_control_ioctl(snd_card_t * card,
 				 snd_control_t * control,
 				 unsigned int cmd, unsigned long arg)
@@ -113,9 +92,6 @@ static int snd_pcm_control_ioctl(snd_card_t * card,
 			return 0;
 		}
 	case SND_CTL_IOCTL_PCM_SUBDEVICE:
-	case SND_CTL_IOCTL_PCM_SWITCH_LIST:
-	case SND_CTL_IOCTL_PCM_SWITCH_READ:
-	case SND_CTL_IOCTL_PCM_SWITCH_WRITE:
 	case SND_CTL_IOCTL_PCM_PREFER_SUBDEVICE:
 	case SND_CTL_IOCTL_PCM_INFO:
 	case SND_CTL_IOCTL_PCM_CHANNEL_INFO:
@@ -142,12 +118,6 @@ static int snd_pcm_control_ioctl(snd_card_t * card,
 				pcm->chn[control->pcm_channel].open_prefer_subchn = val;
 				return 0;
 			}
-		case SND_CTL_IOCTL_PCM_SWITCH_LIST:
-			return snd_switch_list(&pcm->chn[control->pcm_channel].switches, (snd_switch_list_t *) arg);
-		case SND_CTL_IOCTL_PCM_SWITCH_READ:
-			return snd_switch_read(&pcm->chn[control->pcm_channel].switches, pcm, (snd_switch_t *) arg);
-		case SND_CTL_IOCTL_PCM_SWITCH_WRITE:
-			return snd_pcm_switch_write(control, pcm, control->pcm_channel == SND_PCM_CHANNEL_PLAYBACK ? SND_CTL_IFACE_PCM_PLAYBACK : SND_CTL_IFACE_PCM_CAPTURE, &pcm->chn[control->pcm_channel].switches, (snd_switch_t *) arg);
 		case SND_CTL_IOCTL_PCM_INFO:
 			return snd_pcm_info(pcm, (snd_pcm_info_t *) arg);
 		case SND_CTL_IOCTL_PCM_CHANNEL_INFO:
@@ -179,7 +149,7 @@ static int snd_pcm_new_channel(snd_pcm_t *pcm,
 	int idx;
 	snd_pcm_subchn_t *subchn, *prev;
 
-	snd_switch_prepare(&pchn->switches);
+	snd_switch_prepare(pcm->card, &pchn->switches, pcm, SND_CTL_IFACE_PCM, pcm->device, channel);
 #ifdef CONFIG_SND_OSSEMUL
 	init_MUTEX(&pchn->oss.setup_mutex);
 #endif
@@ -254,7 +224,7 @@ static void snd_pcm_free_channel(snd_pcm_channel_t * pchn)
 	snd_pcm_oss_setup_t *setup, *setupn;
 #endif
 
-	snd_switch_free(&pchn->switches);
+	snd_switch_free(pchn->pcm->card, &pchn->switches);
 	subchn = pchn->subchn;
 	while (subchn) {
 		subchn_next = subchn->next;
@@ -518,11 +488,11 @@ int snd_pcm_switch_add(snd_pcm_channel_t * pchn, snd_kswitch_t * ksw)
 	int err = snd_switch_add(&pchn->switches, ksw);
 	if (err >= 0)
 		snd_control_notify_switch_change(pchn->pcm->card,
-					SND_CTL_READ_SWITCH_ADD,
-					pchn->channel == SND_PCM_CHANNEL_PLAYBACK ?
-						SND_CTL_IFACE_PCM_PLAYBACK :
-						SND_CTL_IFACE_PCM_CAPTURE,
-					ksw->name);
+						 SND_CTL_READ_SWITCH_ADD,
+						 SND_CTL_IFACE_PCM,
+						 pchn->pcm->device,
+						 pchn->channel,
+						 ksw->name);
 	return err;
 }
 
@@ -531,11 +501,11 @@ int snd_pcm_switch_remove(snd_pcm_channel_t * pchn, snd_kswitch_t * ksw)
 	int err = snd_switch_remove(&pchn->switches, ksw);
 	if (err >= 0)
 		snd_control_notify_switch_change(pchn->pcm->card,
-					SND_CTL_READ_SWITCH_REMOVE,
-					pchn->channel == SND_PCM_CHANNEL_PLAYBACK ?
-						SND_CTL_IFACE_PCM_PLAYBACK :
-						SND_CTL_IFACE_PCM_CAPTURE,
-					ksw->name);
+						 SND_CTL_READ_SWITCH_REMOVE,
+						 SND_CTL_IFACE_PCM,
+						 pchn->pcm->device,
+						 pchn->channel,
+						 ksw->name);
 	return err;
 }
 
@@ -558,11 +528,11 @@ int snd_pcm_switch_change(snd_pcm_channel_t * pchn, snd_kswitch_t * ksw)
 {
 	snd_debug_check(pchn == NULL || ksw == NULL, -ENXIO);
 	snd_control_notify_switch_change(pchn->pcm->card,
-				SND_CTL_READ_SWITCH_CHANGE,
-				pchn->channel == SND_PCM_CHANNEL_PLAYBACK ?
-					SND_CTL_IFACE_PCM_PLAYBACK :
-					SND_CTL_IFACE_PCM_CAPTURE,
-				ksw->name);
+					 SND_CTL_READ_SWITCH_CHANGE,
+					 SND_CTL_IFACE_PCM,
+					 pchn->pcm->device,
+					 pchn->channel,
+					 ksw->name);
 	return 0;
 }
 
