@@ -386,14 +386,19 @@ static int snd_pcm_lib_ioctl_status(void *private_data,
 		}
 		ptr = runtime->hw->pointer(private_data, subchn);
 		if (!(runtime->flags & SND_PCM_FLG_OSS_MMAP)) {
-			status->count = runtime->frag_used * runtime->buf.block.frag_size;
-			status->free = (runtime->frags - runtime->frag_used) * runtime->buf.block.frag_size;
-			if (subchn->channel == SND_PCM_CHANNEL_PLAYBACK) {
-				status->count -= ptr % runtime->buf.block.frag_size;
-				if (status->count < 0)
-					status->count = 0;
+			if (runtime->flags & SND_PCM_FLG_OSS_MMAP) {
+				status->count = status->free = 0;
+				status->scount += ptr % runtime->buf.block.frag_size;
 			} else {
-				status->count += ptr % runtime->buf.block.frag_size;
+				status->count = runtime->frag_used * runtime->buf.block.frag_size;
+				status->free = (runtime->frags - runtime->frag_used) * runtime->buf.block.frag_size;
+				if (subchn->channel == SND_PCM_CHANNEL_PLAYBACK) {
+					status->count -= ptr % runtime->buf.block.frag_size;
+					if (status->count < 0)
+						status->count = 0;
+				} else {
+					status->count += ptr % runtime->buf.block.frag_size;
+				}
 			}
 		} else {
 			status->count = runtime->interrupts;
@@ -666,7 +671,15 @@ int snd_pcm_playback_data(snd_pcm_subchn_t *subchn)
 		return runtime->interrupts > 0 ? 1 : 0;
 	spin_lock_irqsave(&runtime->lock, flags);
 	if (runtime->mode == SND_PCM_MODE_BLOCK) {
-		result = runtime->frag_used > 0;
+		if (runtime->flags & SND_PCM_FLG_MMAP) {
+			if (*runtime->status == SND_PCM_STATUS_UNDERRUN) {
+				result = 0;
+			} else {
+				result = runtime->mmap_control->fragments[*runtime->frag_tail].data ? 1 : 0;
+			}
+		} else {
+			result = runtime->frag_used > 0;
+		}
 	} else {
 		ptr = snd_pcm_lib_get_stream_ptr(subchn, runtime);
 		result = runtime->position > ptr;
@@ -1096,7 +1109,7 @@ static int snd_pcm_lib_write_transfer(snd_pcm_subchn_t *subchn,
 	snd_pcm_runtime_t *runtime = subchn->runtime;
 	int err;
 	if (runtime->hw_memcpy) {
-		if ((err = runtime->hw_memcpy(subchn, -1, dstoff, (const char *) src + srcoff, size)) < 0)
+		if ((err = runtime->hw_memcpy(subchn, -1, dstoff, (char *) src + srcoff, size)) < 0)
 			return err;
 	} else {
 		if (copy_from_user(runtime->dma_area->buf + dstoff, (const char *) src + srcoff, size))
