@@ -226,8 +226,12 @@ static int snd_pcm_channel_prepare(snd_pcm_subchn_t * subchn)
 	
 	snd_debug_check(subchn == NULL, -ENODEV);
 	runtime = subchn->runtime;
-	if (*runtime->status == SND_PCM_STATUS_RUNNING)
+	switch (*runtime->status) {
+	case SND_PCM_STATUS_NOTREADY:
+		return -EBADFD;
+	case SND_PCM_STATUS_RUNNING:
 		return -EBUSY;
+	}
 	res = runtime->hw->prepare(subchn->pchn->private_data, subchn);
 	if (res >= 0) {
 #ifdef CONFIG_SND_OSSEMUL
