@@ -649,30 +649,6 @@ static int snd_rawmidi_ioctl(struct file *file,
 	return -EINVAL;
 }
 
-static int snd_rawmidi_switch_write(snd_control_t * control,
-				    snd_rawmidi_t * rmidi,
-				    snd_kswitch_list_t * klist,
-				    int iface,
-				    snd_switch_t * _sw)
-{
-	int result;
-	
-	if (snd_control_busy(control))
-		return -EBUSY;
-	result = snd_switch_write(klist, rmidi, _sw);
-	if (result > 0) {
-		snd_switch_t sw;
-
-		if (copy_from_user(&sw, _sw, sizeof(*_sw)))
-			return -EFAULT;
-		snd_control_notify_switch_value_change(control,
-						       iface,
-						       sw.name,
-						       0);		
-	}
-	return -EIO;
-}
-
 int snd_rawmidi_control_ioctl(snd_card_t * card, snd_control_t * control,
 			      unsigned int cmd, unsigned long arg)
 {
@@ -723,21 +699,12 @@ int snd_rawmidi_control_ioctl(snd_card_t * card, snd_control_t * control,
 			return 0;
 		}
 	case SND_CTL_IOCTL_RAWMIDI_INFO:
-	case SND_CTL_IOCTL_RAWMIDI_SWITCH_LIST:
-	case SND_CTL_IOCTL_RAWMIDI_SWITCH_READ:
-	case SND_CTL_IOCTL_RAWMIDI_SWITCH_WRITE:
 		rmidi = snd_rawmidi_devices[tmp + control->rawmidi_device];
 		if (rmidi == NULL)
 			return -ENXIO;
 		switch (cmd) {
 		case SND_CTL_IOCTL_RAWMIDI_INFO:
 			return snd_rawmidi_info(rmidi, (snd_rawmidi_info_t *) arg);
-		case SND_CTL_IOCTL_RAWMIDI_SWITCH_LIST:
-			return snd_switch_list(&rmidi->chn[control->rawmidi_channel].switches, (snd_switch_list_t *) arg);
-		case SND_CTL_IOCTL_RAWMIDI_SWITCH_READ:
-			return snd_switch_read(&rmidi->chn[control->rawmidi_channel].switches, rmidi, (snd_switch_t *) arg);
-		case SND_CTL_IOCTL_RAWMIDI_SWITCH_WRITE:
-			return snd_rawmidi_switch_write(control, rmidi, &rmidi->chn[control->rawmidi_channel].switches, SND_CTL_IFACE_RAWMIDI_OUTPUT, (snd_switch_t *) arg);
 		}
 		break;
 	}
@@ -1121,8 +1088,8 @@ int snd_rawmidi_new(snd_card_t * card, char *id, int device, snd_rawmidi_t ** rr
 	init_waitqueue_head(&ochn->sleep);
 	init_MUTEX(&rmidi->open_mutex);
 	init_waitqueue_head(&rmidi->open_wait);
-	snd_switch_prepare(&ichn->switches);
-	snd_switch_prepare(&ochn->switches);
+	snd_switch_prepare(card, &ichn->switches, rmidi, SND_CTL_IFACE_RAWMIDI, device, SND_RAWMIDI_CHANNEL_INPUT);
+	snd_switch_prepare(card, &ochn->switches, rmidi, SND_CTL_IFACE_RAWMIDI, device, SND_RAWMIDI_CHANNEL_OUTPUT);
 	ichn->timer.data = (unsigned long) rmidi;
 	ichn->timer.function = snd_rawmidi_poll_input;
 	ochn->timer.data = (unsigned long) rmidi;
@@ -1144,8 +1111,8 @@ static int snd_rawmidi_free(snd_rawmidi_t * rmidi)
 	snd_debug_check(rmidi == NULL, -ENXIO);
 	ichn = &rmidi->chn[SND_RAWMIDI_CHANNEL_INPUT];
 	ochn = &rmidi->chn[SND_RAWMIDI_CHANNEL_OUTPUT];
-	snd_switch_free(&ochn->switches);
-	snd_switch_free(&ichn->switches);
+	snd_switch_free(rmidi->card, &ochn->switches);
+	snd_switch_free(rmidi->card, &ichn->switches);
 	if (ochn->hw.private_free)
 		ochn->hw.private_free(ochn->hw.private_data);
 	if (ichn->hw.private_free)
