@@ -635,28 +635,12 @@ static int snd_virmidi_control_ioctl(snd_card_t *card, snd_control_t *control,
 			return 0;
 		}
 	case SND_CTL_IOCTL_RAWMIDI_INFO:
-	case SND_CTL_IOCTL_RAWMIDI_SWITCH_LIST:
-	case SND_CTL_IOCTL_RAWMIDI_SWITCH_READ:
-	case SND_CTL_IOCTL_RAWMIDI_SWITCH_WRITE:
 		rdev = snd_virmidi_devices[tmp + control->rawmidi_device];
 		if (rdev == NULL)
 			return -ENXIO;
 		switch (cmd) {
 		case SND_CTL_IOCTL_RAWMIDI_INFO:
 			return snd_virmidi_dev_info(rdev, (snd_rawmidi_info_t *) arg);
-#ifdef SUPPORT_SWITCHES
-		case SND_CTL_IOCTL_RAWMIDI_SWITCH_LIST:
-			return snd_switch_list(&rdev->switches[control->rawmidi_channel], (snd_switch_list_t *) arg);
-		case SND_CTL_IOCTL_RAWMIDI_SWITCH_READ:
-			return snd_switch_read(&rdev->switches[control->rawmidi_channel], rdev, (snd_switch_t *) arg);
-		case SND_CTL_IOCTL_RAWMIDI_SWITCH_WRITE:
-			return snd_virmidi_dev_switch_write(control, rdev, &rdev->switches[control->rawmidi_channel], SND_CTL_IFACE_RAWMIDI_OUTPUT, (snd_switch_t *) arg);
-#else
-		case SND_CTL_IOCTL_RAWMIDI_SWITCH_LIST:
-		case SND_CTL_IOCTL_RAWMIDI_SWITCH_READ:
-		case SND_CTL_IOCTL_RAWMIDI_SWITCH_WRITE:
-			return 0;
-#endif
 		}
 		break;
 	}
@@ -1096,8 +1080,8 @@ int snd_virmidi_new(snd_card_t *card, int device, snd_virmidi_dev_t **rrdev)
 	rdev->files = 0;
 	rdev->filelist = NULL;
 	
-	snd_switch_prepare(&rdev->switches[SND_RAWMIDI_CHANNEL_INPUT]);
-	snd_switch_prepare(&rdev->switches[SND_RAWMIDI_CHANNEL_OUTPUT]);
+	snd_switch_prepare(card, &rdev->switches[SND_RAWMIDI_CHANNEL_INPUT], rdev, SND_CTL_IFACE_RAWMIDI, device, SND_RAWMIDI_CHANNEL_INPUT);
+	snd_switch_prepare(card, &rdev->switches[SND_RAWMIDI_CHANNEL_OUTPUT], rdev, SND_CTL_IFACE_RAWMIDI, device, SND_RAWMIDI_CHANNEL_OUTPUT);
 	if ((err = snd_device_new(card, SND_DEV_RAWMIDI, rdev, device, &ops, NULL)) < 0) {
 		snd_virmidi_dev_free(rdev);
 		return err;
@@ -1113,8 +1097,8 @@ int snd_virmidi_new(snd_card_t *card, int device, snd_virmidi_dev_t **rrdev)
 static int snd_virmidi_dev_free(snd_virmidi_dev_t *rdev)
 {
 	snd_debug_check(rdev == NULL, -ENXIO);
-	snd_switch_free(&rdev->switches[SND_RAWMIDI_CHANNEL_INPUT]);
-	snd_switch_free(&rdev->switches[SND_RAWMIDI_CHANNEL_OUTPUT]);
+	snd_switch_free(rdev->card, &rdev->switches[SND_RAWMIDI_CHANNEL_INPUT]);
+	snd_switch_free(rdev->card, &rdev->switches[SND_RAWMIDI_CHANNEL_OUTPUT]);
 	if (rdev->private_free)
 		rdev->private_free(rdev->private_data);
 	snd_magic_kfree(rdev);
