@@ -659,20 +659,14 @@ EXPORT_SYMBOL(snd_info_unregister);
 EXPORT_SYMBOL(snd_oss_info_register);
 #endif
   /* switch.c */
+EXPORT_SYMBOL(snd_switch_lock);
 EXPORT_SYMBOL(snd_switch_prepare);
 EXPORT_SYMBOL(snd_switch_new);
 EXPORT_SYMBOL(snd_switch_free_one);
 EXPORT_SYMBOL(snd_switch_add);
 EXPORT_SYMBOL(snd_switch_remove);
 EXPORT_SYMBOL(snd_switch_free);
-EXPORT_SYMBOL(snd_switch_lock);
-EXPORT_SYMBOL(snd_switch_list);
-EXPORT_SYMBOL(snd_switch_read);
-EXPORT_SYMBOL(snd_switch_write);
 EXPORT_SYMBOL(snd_switch_count);
-EXPORT_SYMBOL(snd_switch_size);
-EXPORT_SYMBOL(snd_switch_store);
-EXPORT_SYMBOL(snd_switch_restore);
   /* control.c */
 EXPORT_SYMBOL(snd_control_busy);
 EXPORT_SYMBOL(snd_control_notify_structure_change);
