@@ -42,6 +42,7 @@ static struct {
 	{	0x414B4D00,	"Asahi Kasei AK4540"	},
 	{	0x41445303,	"Analog Devices AD1819"	},
 	{	0x41445340,	"Analog Devices AD1881"	},
+	{	0x41445448,	"Analog Devices AD1885?" },
 	{	0x43525903,	"Cirrus Logic CS4297"   },
 	{	0x43525913,	"Cirrus Logic CS4297A"	},
 	{	0x42525923,	"Cirrus Logic CS4294"	},
@@ -105,7 +106,9 @@ static int snd_ac97_valid_reg(ac97_t *ac97, unsigned short reg)
 		if (reg <= 0x1c || reg == 0x20 || reg == 0x26 || reg >= 0x7c)
 			return 1;
 		return 0;
+	case 0x41445303:	/* AD1819 */
 	case 0x41445340:	/* AD1881 */
+	case 0x41445348:	/* AD1885? */
 		if (reg >= 0x3a && reg <= 0x59)
 			return 0;
 		return 1;
@@ -136,6 +139,16 @@ void snd_ac97_write_lock(ac97_t *ac97, unsigned short reg, unsigned short value)
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
 }
 
+void snd_ac97_write_bitmask_lock(ac97_t *ac97, unsigned short reg,
+				 unsigned short mask, unsigned short value)
+{
+	unsigned long flags;
+
+	spin_lock_irqsave(&ac97->reg_lock, flags);
+	snd_ac97_write(ac97, reg, (snd_ac97_read(ac97, reg) & mask) | value);
+	spin_unlock_irqrestore(&ac97->reg_lock, flags);
+}
+
 unsigned short snd_ac97_read_lock(ac97_t *ac97, unsigned short reg)
 {
 	unsigned long flags;
@@ -1903,7 +1916,7 @@ int snd_ac97_mixer(snd_card_t * card, int device,
 	snd_ac97_write(ac97, AC97_POWERDOWN, 0);	/* nothing should be in powerdown mode */
 	snd_ac97_write(ac97, AC97_GENERAL_PURPOSE, 0);
 	if (ac97->init)
-		ac97->init(ac97->private_data);
+		ac97->init(ac97->private_data, ac97);
 	if (snd_ac97_mixer_build(mixer, ac97, pcm_count, pcm_dev) < 0) {
 		snd_device_free(card, mixer);
 		return -ENOMEM;
@@ -2214,6 +2227,7 @@ static void snd_ac97_chip_init(ac97_t * ac97)
 EXPORT_SYMBOL(snd_ac97_write);
 EXPORT_SYMBOL(snd_ac97_read);
 EXPORT_SYMBOL(snd_ac97_write_lock);
+EXPORT_SYMBOL(snd_ac97_write_bitmask_lock);
 EXPORT_SYMBOL(snd_ac97_read_lock);
 EXPORT_SYMBOL(snd_ac97_mixer);
 
