@@ -541,7 +541,7 @@ static snd_pcm_hardware_t snd_es1688_playback =
 	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
 	SND_PCM_CHNINFO_MMAP_VALID,	/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	4000,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
@@ -563,7 +563,7 @@ static snd_pcm_hardware_t snd_es1688_capture =
 	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
 	SND_PCM_CHNINFO_MMAP_VALID,	/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	4000,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
